@@ -768,6 +768,8 @@ typedef enum _D3DKMT_ESCAPETYPE
     D3DKMT_ESCAPE_SETDRIVERUPDATESTATUS,
     D3DKMT_ESCAPE_DRT_TEST,
     D3DKMT_ESCAPE_DIAGNOSTICS,
+    /* Wine-specific escape codes */
+    D3DKMT_ESCAPE_UPDATE_RESOURCE_WINE = 0x80000000,
 } D3DKMT_ESCAPETYPE;
 
 typedef struct _D3DKMT_ESCAPE
@@ -810,22 +812,6 @@ typedef struct _D3DKMT_CREATEKEYEDMUTEX
     D3DKMT_HANDLE hKeyedMutex;
 } D3DKMT_CREATEKEYEDMUTEX;
 
-typedef struct _D3DDDICB_SIGNALFLAGS
-{
-    union
-    {
-        struct
-        {
-            UINT SignalAtSubmission : 1;
-            UINT EnqueueCpuEvent : 1;
-            UINT AllowFenceRewind : 1;
-            UINT Reserved : 28;
-            UINT DXGK_SIGNAL_FLAG_INTERNAL0 : 1;
-        };
-        UINT Value;
-    };
-} D3DDDICB_SIGNALFLAGS;
-
 typedef struct _D3DKMT_CREATEKEYEDMUTEX2_FLAGS
 {
     union
@@ -876,6 +862,40 @@ typedef struct _D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE
     UINT PrivateRuntimeDataSize;
 } D3DKMT_OPENKEYEDMUTEXFROMNTHANDLE;
 
+typedef struct _D3DKMT_ACQUIREKEYEDMUTEX
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    LARGE_INTEGER *pTimeout;
+    UINT64 FenceValue;
+} D3DKMT_ACQUIREKEYEDMUTEX;
+
+typedef struct _D3DKMT_RELEASEKEYEDMUTEX
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    UINT64 FenceValue;
+} D3DKMT_RELEASEKEYEDMUTEX;
+
+typedef struct _D3DKMT_ACQUIREKEYEDMUTEX2
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    LARGE_INTEGER *pTimeout;
+    UINT64 FenceValue;
+    VOID *pPrivateRuntimeData;
+    UINT PrivateRuntimeDataSize;
+} D3DKMT_ACQUIREKEYEDMUTEX2;
+
+typedef struct _D3DKMT_RELEASEKEYEDMUTEX2
+{
+    D3DKMT_HANDLE hKeyedMutex;
+    UINT64 Key;
+    UINT64 FenceValue;
+    VOID *pPrivateRuntimeData;
+    UINT PrivateRuntimeDataSize;
+} D3DKMT_RELEASEKEYEDMUTEX2;
+
 typedef struct _D3DKMT_OPENNTHANDLEFROMNAME
 {
     DWORD dwDesiredAccess;
@@ -883,68 +903,6 @@ typedef struct _D3DKMT_OPENNTHANDLEFROMNAME
     HANDLE hNtHandle;
 } D3DKMT_OPENNTHANDLEFROMNAME;
 
-typedef ULONGLONG D3DGPU_VIRTUAL_ADDRESS;
-
-#ifndef D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS_EXT
-#define D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS_EXT
-#define D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS_RESERVED0 Reserved0
-#endif
-
-typedef struct _D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS
-{
-    union
-    {
-        struct
-        {
-            UINT Shared : 1;
-            UINT NtSecuritySharing : 1;
-            UINT CrossAdapter : 1;
-            UINT TopOfPipeline : 1;
-            UINT NoSignal : 1;
-            UINT NoWait : 1;
-            UINT NoSignalMaxValueOnTdr : 1;
-            UINT NoGPUAccess : 1;
-            UINT Reserved : 23;
-            UINT D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS_RESERVED0 : 1;
-        };
-        UINT Value;
-    };
-} D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS;
-
-typedef UINT D3DDDI_VIDEO_PRESENT_TARGET_ID;
-
-typedef enum _D3DDDI_SYNCHRONIZATIONOBJECT_TYPE
-{
-    D3DDDI_SYNCHRONIZATION_MUTEX = 1,
-    D3DDDI_SEMAPHORE = 2,
-    D3DDDI_FENCE = 3,
-    D3DDDI_CPU_NOTIFICATION = 4,
-    D3DDDI_MONITORED_FENCE = 5,
-    D3DDDI_PERIODIC_MONITORED_FENCE = 6,
-    D3DDDI_SYNCHRONIZATION_TYPE_LIMIT
-} D3DDDI_SYNCHRONIZATIONOBJECT_TYPE;
-
-typedef struct _D3DDDI_SYNCHRONIZATIONOBJECTINFO
-{
-    D3DDDI_SYNCHRONIZATIONOBJECT_TYPE Type;
-    union
-    {
-        struct
-        {
-            BOOL InitialState;
-        } SynchronizationMutex;
-        struct
-        {
-            UINT MaxCount;
-            UINT InitialCount;
-        } Semaphore;
-        struct
-        {
-            UINT Reserved[16];
-        } Reserved;
-    };
-} D3DDDI_SYNCHRONIZATIONOBJECTINFO;
-
 typedef struct _D3DKMT_CREATESYNCHRONIZATIONOBJECT
 {
     D3DKMT_HANDLE hDevice;
@@ -952,53 +910,6 @@ typedef struct _D3DKMT_CREATESYNCHRONIZATIONOBJECT
     D3DKMT_HANDLE hSyncObject;
 } D3DKMT_CREATESYNCHRONIZATIONOBJECT;
 
-typedef struct _D3DDDI_SYNCHRONIZATIONOBJECTINFO2
-{
-    D3DDDI_SYNCHRONIZATIONOBJECT_TYPE Type;
-    D3DDDI_SYNCHRONIZATIONOBJECT_FLAGS Flags;
-    union
-    {
-        struct
-        {
-            BOOL InitialState;
-        } SynchronizationMutex;
-        struct
-        {
-            UINT MaxCount;
-            UINT InitialCount;
-        } Semaphore;
-        struct
-        {
-            UINT64 FenceValue;
-        } Fence;
-        struct
-        {
-            HANDLE Event;
-        } CPUNotification;
-        struct
-        {
-            UINT64 InitialFenceValue;
-            void *FenceValueCPUVirtualAddress;
-            D3DGPU_VIRTUAL_ADDRESS FenceValueGPUVirtualAddress;
-            UINT EngineAffinity;
-        } MonitoredFence;
-        struct
-        {
-            D3DKMT_HANDLE hAdapter;
-            D3DDDI_VIDEO_PRESENT_TARGET_ID VidPnTargetId;
-            UINT64 Time;
-            void *FenceValueCPUVirtualAddress;
-            D3DGPU_VIRTUAL_ADDRESS FenceValueGPUVirtualAddress;
-            UINT EngineAffinity;
-        } PeriodicMonitoredFence;
-        struct
-        {
-            UINT64 Reserved[8];
-        } Reserved;
-    };
-    D3DKMT_HANDLE SharedHandle;
-} D3DDDI_SYNCHRONIZATIONOBJECTINFO2;
-
 typedef struct _D3DKMT_CREATESYNCHRONIZATIONOBJECT2
 {
     D3DKMT_HANDLE hDevice;
@@ -1050,6 +961,25 @@ typedef struct _D3DKMT_DESTROYSYNCHRONIZATIONOBJECT
     D3DKMT_HANDLE hSyncObject;
 } D3DKMT_DESTROYSYNCHRONIZATIONOBJECT;
 
+typedef struct _D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU
+{
+    D3DKMT_HANDLE hDevice;
+    UINT ObjectCount;
+    const D3DKMT_HANDLE *ObjectHandleArray;
+    const UINT64 *FenceValueArray;
+    D3DDDICB_SIGNALFLAGS Flags;
+} D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU;
+
+typedef struct _D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU
+{
+    D3DKMT_HANDLE hDevice;
+    UINT ObjectCount;
+    const D3DKMT_HANDLE *ObjectHandleArray;
+    const UINT64 *FenceValueArray;
+    HANDLE hAsyncEvent;
+    D3DDDI_WAITFORSYNCHRONIZATIONOBJECTFROMCPU_FLAGS Flags;
+} D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU;
+
 typedef struct _D3DKMT_CREATESTANDARDALLOCATIONFLAGS
 {
     union
@@ -1293,6 +1223,8 @@ extern "C"
 {
 #endif /* __cplusplus */
 
+NTSTATUS WINAPI D3DKMTAcquireKeyedMutex( D3DKMT_ACQUIREKEYEDMUTEX *params );
+NTSTATUS WINAPI D3DKMTAcquireKeyedMutex2( D3DKMT_ACQUIREKEYEDMUTEX2 *params );
 NTSTATUS WINAPI D3DKMTCheckOcclusion( const D3DKMT_CHECKOCCLUSION *desc );
 NTSTATUS WINAPI D3DKMTCheckVidPnExclusiveOwnership(const D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP *desc);
 NTSTATUS WINAPI D3DKMTCloseAdapter(const D3DKMT_CLOSEADAPTER *desc);
@@ -1332,9 +1264,14 @@ NTSTATUS WINAPI D3DKMTQueryResourceInfo( D3DKMT_QUERYRESOURCEINFO *params );
 NTSTATUS WINAPI D3DKMTQueryResourceInfoFromNtHandle( D3DKMT_QUERYRESOURCEINFOFROMNTHANDLE *params );
 NTSTATUS WINAPI D3DKMTQueryStatistics(D3DKMT_QUERYSTATISTICS *stats);
 NTSTATUS WINAPI D3DKMTQueryVideoMemoryInfo(D3DKMT_QUERYVIDEOMEMORYINFO *desc);
+NTSTATUS WINAPI D3DKMTReleaseKeyedMutex( D3DKMT_RELEASEKEYEDMUTEX *params );
+NTSTATUS WINAPI D3DKMTReleaseKeyedMutex2( D3DKMT_RELEASEKEYEDMUTEX2 *params );
 NTSTATUS WINAPI D3DKMTSetQueuedLimit(D3DKMT_SETQUEUEDLIMIT *desc);
 NTSTATUS WINAPI D3DKMTSetVidPnSourceOwner(const D3DKMT_SETVIDPNSOURCEOWNER *desc);
 NTSTATUS WINAPI D3DKMTShareObjects( UINT count, const D3DKMT_HANDLE *handles, OBJECT_ATTRIBUTES *attr, UINT access, HANDLE *handle );
+NTSTATUS WINAPI D3DKMTSignalSynchronizationObjectFromCpu( const D3DKMT_SIGNALSYNCHRONIZATIONOBJECTFROMCPU *params );
+NTSTATUS WINAPI D3DKMTWaitForSynchronizationObjectFromCpu( const D3DKMT_WAITFORSYNCHRONIZATIONOBJECTFROMCPU *params );
+
 
 #ifdef __cplusplus
 }
